ShivvAudio = LibStub('AceAddon-3.0'):NewAddon('ShivvAudio', 'AceConsole-3.0', 'AceEvent-3.0')

local L = LibStub('AceLocale-3.0'):GetLocale('ShivvAudio')
local defaults = {
	profile = {
		-- global
		trackingAuras = false,
		trackingCombat = false,
		trackingEvents = false,
		trackingVitals = false,
		musicControl = false,

		-- auras
		auras = {},

		-- combat
		combat = {
			tieredCombat = false,
			bgEnable = false,
			outsideEnable = false,
			partyEnable = false,
			raidEnable = false,
			outsideTier2Mobs = 2,
			outsideTier3Mobs = 4,
			partyTier2Mobs = 2,
			partyTier3Mobs = 4,


			media = {
				-- battlegrounds
				bgStart = nil,
				bgEnd = nil,
				bgCombat = nil,
				-- outside (overworld)
				outsideStart = nil,
				outsideEnd = nil,
				outsideCombat = nil,
				outsideElite = nil,
				outsideT1 = nil,
				outsideT2 = nil,
				outsideT3 = nil,
				-- party (dungeons)
				partyStart = nil,
				partyEnd = nil,
				partyBossKill = nil,
				partyBossWipe = nil,
				partyCombat = nil,
				partyBoss = nil,
				partyT1 = nil,
				partyT2 = nil,
				partyT3 = nil,
				-- raid (10-40 man)
				raidStart = nil,
				raidEnd = nil,
				raidCombat = nil,
				raidBoss = nil,
				raidBossKill = nil,
				raidBossWipe = nil,
			},
		},

		-- events
		events = {
			achievement = nil,
			auction = nil,
			bank = nil,
			death = nil,
			levelup = nil,
			lootopen = nil,
			lootclose = nil,
			loot = nil,
			merchant = nil,
			trade = nil,
		},

		-- vitals
		vitals = {
			reset = 5,
			health = {},
			power = {},
		},
	}
}

-- ON ZONE DELAYED TIMER
-- Restarts music (if any) on a zone change.
do
	local handler = CreateFrame('Frame')
	local timer
	local function MusicDelay(frame, elapsed)
		timer = timer - elapsed

		if (timer <= 0) then
			handler:SetScript('OnUpdate', nil)
			ShivvAudio:Play()
		end
	end

	function ShivvAudio:StartOnZoneDelay()
		timer = 1.5
		handler:SetScript('OnUpdate', MusicDelay)
	end
end

-- MOD BASE
function ShivvAudio:OnInitialize()
	self.db = LibStub('AceDB-3.0'):New('ShivvAudioDB', defaults)
	self.db.RegisterCallback(self, 'OnProfileChanged', 'RefreshConfig')
	self.db.RegisterCallback(self, 'OnProfileCopied', 'RefreshConfig')
	self.db.RegisterCallback(self, 'OnProfileReset', 'RefreshConfig')

	-- add global entry if not present (new user)
	if (not self.db.global.mediaDB) then
		self.db.global.mediaDB = {}
	else
		-- previous user, check if using older database
		if (self.db.global.version ~= 3) then
			self:ConvertDatabase()
		end
	end

	self:RegisterChatCommand('shivv', 'ChatHandler')
	self:RegisterEvent('PLAYER_ENTERING_WORLD')
	self:PopulateMusicLookup() -- do initial lookup population
end

function ShivvAudio:OnEnable()
end

function ShivvAudio:OnDisable()
end

function ShivvAudio:RefreshConfig()
	self:GetModule('TrackingAuras'):OnDisable()
	self:GetModule('TrackingCombat'):OnDisable()
	self:GetModule('TrackingEvents'):OnDisable()
	self:GetModule('TrackingVitals'):OnDisable()

	if (self.db.profile.trackingAuras) then
		self:GetModule('TrackingAuras'):OnEnable()
	end

	if (self.db.profile.trackingCombat) then
		self:GetModule('TrackingCombat'):OnEnable()
	end

	if (self.db.profile.trackingEvents) then
		self:GetModule('TrackingEvents'):OnEnable()
	end

	if (self.db.profile.trackingVitals) then
		self:GetModule('TrackingVitals'):OnEnable()
	end

	self:PopulateMusicLookup() -- do initial lookup population
end

-- EVENT HANDLERS
function ShivvAudio:PLAYER_ENTERING_WORLD()
	self:StartOnZoneDelay()
end

-- SOUND HANDLER
function ShivvAudio:SoundHandler(sound)
	if (sound and self.db.global.mediaDB[sound]) then
		PlaySoundFile(self.db.global.mediaDB[sound].path)
	end
end

-- MUSIC HANDLER
local musicStack = {}
local musicLookup = {}

function ShivvAudio:PopulateMusicLookup()
	for k, v in pairs(musicLookup) do
		musicLookup[k] = nil -- clean out current lookup table
	end

	local db = self.db.profile

	-- events
	musicLookup['shivv-auction'] = db.events.auction or nil
	musicLookup['shivv-bank'] = db.events.bank or nil
	musicLookup['shivv-loot'] = db.events.loot or nil
	musicLookup['shivv-merchant'] = db.events.merchant or nil
	musicLookup['shivv-trade'] = db.events.trade or nil

	-- combat
	musicLookup['shivv-bg-combat'] = db.combat.media.bgCombat or nil
	musicLookup['shivv-outside-combat'] = db.combat.media.outsideCombat or nil
	musicLookup['shivv-outside-elite'] = db.combat.media.outsideElite or nil
	musicLookup['shivv-outside-t1'] = db.combat.media.outsideT1 or nil
	musicLookup['shivv-outside-t2'] = db.combat.media.outsideT2 or nil
	musicLookup['shivv-outside-t3'] = db.combat.media.outsideT3 or nil
	musicLookup['shivv-party-combat'] = db.combat.media.partyCombat or nil
	musicLookup['shivv-party-boss'] = db.combat.media.partyBoss or nil
	musicLookup['shivv-party-t1'] = db.combat.media.partyT1 or nil
	musicLookup['shivv-party-t2'] = db.combat.media.partyT2 or nil
	musicLookup['shivv-party-t3'] = db.combat.media.partyT3 or nil
	musicLookup['shivv-raid-combat'] = db.combat.media.raidCombat or nil
	musicLookup['shivv-raid-boss'] = db.combat.media.raidBoss or nil

	-- auras
	if (db.auras.player) then -- only process if there are potential musical entries (ie, on the player)
		for aura, data in pairs(db.auras.player) do
			if (data.music) then -- music is set
				musicLookup[aura] = data.music -- set lookup
			end
		end
	end
end


function ShivvAudio:Play()
	if (musicStack[1]) then -- check stacked music for an entry
		if (self.db.profile.musicControl and GetCVar('Sound_EnableMusic') == '0') then
			SetCVar('Sound_EnableMusic', '1') -- enable cvar if not enabled and we have control
		end

		PlayMusic(self.db.global.mediaDB[musicLookup[musicStack[1]]].path) -- lookup file and play!
	end
end

function ShivvAudio:Stop()
	StopMusic()

	if (self.db.profile.musicControl and GetCVar('Sound_EnableMusic') == '1') then
		SetCVar('Sound_EnableMusic', '0') -- disable cvar if enabled and we have control
	end
end

function ShivvAudio:CheckForActiveMusic()
	return (#musicStack > 0)
end

function ShivvAudio:KillAllMusic()
	for i = #musicStack, 1, 1 do -- clean out music stack
		tremove(musicStack, i)
	end

	self:Stop() -- stop music
end

function ShivvAudio:MusicStart(musicRef)
	if (musicRef and musicLookup[musicRef]) then -- ensure musicRef has a valid lookup before pushing
		tinsert(musicStack, 1, musicRef) -- push to top of stack
		self:Play()
	end
end

local musicRefActive
function ShivvAudio:MusicStop(musicRef)
	if (musicRef) then
		musicRefActive = (musicStack[1] == musicRef) and true or false -- is ref the currently active music?

		for i = #musicStack, 1, -1 do -- reverse iterate and remove any instances of musicRef
			if (musicStack[i] == musicRef) then
				tremove(musicStack, i)
			end
		end

		if (musicRefActive) then -- this ref was the active music, check for new music or stop music
			if (musicStack[1]) then -- found another entry to play instead
				self:Play()
			else
				self:Stop()
			end
		end
	end
end

-- -------------- --
-- MISC FUNCTIONS --
-- -------------- --
function ShivvAudio:ChatHandler(input)
	if (input == '') then
		self:ShowGUI()
	elseif (input == 'stop' or input == 'kill') then
		self:KillAllMusic()
	else
		self:Print(L.ChatHelp)
	end
end

function ShivvAudio:LSMImport()
	local path
	local entriesAdded = 0
	local LSM = LibStub:GetLibrary('LibSharedMedia-3.0', true)

	local media = LSM:List('sound')

	for k, name in pairs(media) do
		path = LSM:Fetch('sound', name)

		if (path == '' or path == nil or path == 'None' or path == 'none' or path == 0 or self.db.global.mediaDB[name]) then
			-- do nothing if bad path, or if the entry already exists
		else
			self.db.global.mediaDB[name] = {
				['isMusic'] = false,
				['path'] = path,
			}
			entriesAdded = entriesAdded + 1
		end
	end

	self:Print(entriesAdded .. L.LSMEntriesAdded)
end

function ShivvAudio:RemoveMedia(media)
	local changed = 0

	self:Print(L.RemovingMedia)

	for char, charData in pairs(ShivvAudioDB.profiles) do
		-- first: auras
		if (charData.auras) then
			for unit in pairs(charData.auras) do
				for buff in pairs(charData.auras[unit]) do
					if (charData.auras[unit][buff].onGain and charData.auras[unit][buff].onGain == media) then
						charData.auras[unit][buff].onGain = nil
						changed = changed + 1
					end
					if (charData.auras[unit][buff].onLoss and charData.auras[unit][buff].onLoss == media) then
						charData.auras[unit][buff].onLoss = nil
						changed = changed + 1
					end
					if (charData.auras[unit][buff].music and charData.auras[unit][buff].music == media) then
						charData.auras[unit][buff].music = nil
						changed = changed + 1
					end
				end
			end
		end

		-- then combat
		if (charData.combat and charData.combat.media) then
			for m in pairs(charData.combat.media) do
				if (charData.combat.media[m] and charData.combat.media[m] == media) then
					charData.combat.media[m] = nil
					changed = changed + 1
				end
			end
		end

		-- then vitals
		if (charData.vitals) then
			if (charData.vitals.health) then
				for unit in pairs(charData.vitals.health) do
					if (charData.vitals.health[unit].sound and charData.vitals.health[unit].sound == media) then
						charData.vitals.health[unit].sound = nil
						changed = changed + 1
					end
				end
			end

			if (charData.vitals.power) then
				for unit in pairs(charData.vitals.power) do
					if (charData.vitals.power[unit].sound and charData.vitals.power[unit].sound == media) then
						charData.vitals.power[unit].sound = nil
						changed = changed + 1
					end
				end
			end
		end

		-- then events
		if (charData.events) then
			for event in pairs(charData.events) do
				if (charData.events[event] == media) then
					charData.musicalEvents[event] = nil
					changed = changed + 1
				end
			end
		end

		self:Print(format(L.RemovingMediaForChar, changed, char))
	end

	if (self.db.profile.trackingEvents) then
		self:GetModule('TrackingEvents'):OnDisable()
		self:GetModule('TrackingEvents'):OnEnable()
	end

	if (self.db.profile.trackingVitals) then
		self:GetModule('TrackingVitals'):OnDisable()
		self:GetModule('TrackingVitals'):OnEnable()
	end

	self:PopulateMusicLookup() -- repopulate the music lookup

	-- and finally delete the actual entry
	self.db.global.mediaDB[media] = nil
end

-- DATABASE CONVERSION
function ShivvAudio:ConvertDatabase()
	self:Print(L.ConvertDB1)

	for char, data in pairs(ShivvAudioDB.profiles) do
		-- base settings
		if (data.enableAudibleAuras) then
			data.trackingAuras = true
		end

		if (data.enableSymphoniousCombat) then
			data.trackingCombat = true
		end

		if (data.enableMusicalEvents) then
			data.trackingEvents = true
		end

		if (data.enableMelodicVitals) then
			data.trackingVitals = true
		end

		if (data.gameMusicControl) then
			data.musicControl = true
		end

		data.enableAudibleAuras = nil
		data.enableMelodicVitals = nil
		data.enableMusicalEvents = nil
		data.enableSymphoniousCombat = nil
		data.gameMusicControl = nil

		if (data.audibleAuras) then
			data.auras = {}

			for unit, auras in pairs(data.audibleAuras) do
				data.auras[unit] = auras

				for aura in pairs(data.auras[unit]) do
					if (data.auras[unit][aura].onGain == 0) then data.auras[unit][aura].onGain = nil end
					if (data.auras[unit][aura].onLoss == 0) then data.auras[unit][aura].onLoss = nil end
					if (data.auras[unit][aura].music == 0) then data.auras[unit][aura].music = nil end
				end
			end

			data.audibleAuras = nil
		end

		if (data.melodicVitals) then
			data.vitals = {
				reset = data.melodicVitals.repeatTimer or 5
			}

			if (data.melodicVitals.healthDB) then
				data.vitals.health = {}

				for k, v in pairs(data.melodicVitals.healthDB) do
					data.vitals.health[k] = v

					if (data.vitals.health[k].sound and data.vitals.health[k].sound == 0) then data.vitals.health[k].sound = nil end
				end
			end

			if (data.melodicVitals.powerDB) then
				data.vitals.power = {}

				for k, v in pairs(data.melodicVitals.powerDB) do
					data.vitals.power[k] = v

					if (data.vitals.power[k].sound and data.vitals.power[k].sound == 0) then data.vitals.power[k].sound = nil end
				end
			end

			data.melodicVitals = nil
		end

		if (data.musicalEvents) then
			data.events = {
				achievement = data.musicalEvents.achievement or nil,
				auction = data.musicalEvents.auction or nil,
				bank = data.musicalEvents.bank or nil,
				death = data.musicalEvents.death or nil,
				levelup = data.musicalEvents.levelup or nil,
				lootopen = data.musicalEvents.lootopen or nil,
				lootclose = data.musicalEvents.lootclose or nil,
				loot = data.musicalEvents.looting or nil,
				merchant = data.musicalEvents.shop or nil,
				trade = data.musicalEvents.crafting or nil,
			}

			if (data.events.achievement == 0) then data.events.achievement = nil end
			if (data.events.auction == 0) then data.events.auction = nil end
			if (data.events.bank == 0) then data.events.bank = nil end
			if (data.events.death == 0) then data.events.death = nil end
			if (data.events.levelup == 0) then data.events.levelup = nil end
			if (data.events.lootopen == 0) then data.events.lootopen = nil end
			if (data.events.lootclose == 0) then data.events.lootclose = nil end
			if (data.events.loot == 0) then data.events.loot = nil end
			if (data.events.merchant == 0) then data.events.merchant = nil end
			if (data.events.trade == 0) then data.events.trade = nil end

			data.musicalEvents = nil
		end

		if (data.symphoniousCombat) then
			data.symphoniousCombat = nil
		end

		if (data.versionMajor) then
			data.versionMajor = nil
		end
	end

	self:RefreshConfig()
	self:Print(L.ConvertDB2)
	self.db.global.version = 3 -- add new version check for any future db changes
end
