local MODNAME = 'TrackingCombat'
local ShivvAudio = LibStub('AceAddon-3.0'):GetAddon('ShivvAudio')
local TrackingCombat = ShivvAudio:NewModule(MODNAME, 'AceEvent-3.0')

local db
local DBMEnabled, inDBMFight, inShivvFight = false, false, false
local knownMobs, activeMobs, eliteMobs, inTier = {}, 0, {}, 1
local activeArea, activeAreaID, inDungeon, inOverworld = false, '', false, false

function TrackingCombat:OnInitialize()

	self:SetEnabledState(ShivvAudio.db.profile.trackingCombat)
end

function TrackingCombat:OnEnable()
	if (DBM and not DBMEnabled) then
		DBMEnabled = true -- only do this one no matter how many enables/disables
		DBM:RegisterCallback('pull', self.DBMOnPull)
		DBM:RegisterCallback('wipe', self.DBMOnWipe)
		DBM:RegisterCallback('kill', self.DBMOnKill)
	end

	db = ShivvAudio.db.profile.combat -- local db ref

	self:RegisterEvent('PLAYER_ENTERING_WORLD')
	self:RegisterEvent('PLAYER_REGEN_DISABLED')
	self:RegisterEvent('PLAYER_REGEN_ENABLED')

	self:PLAYER_ENTERING_WORLD() -- call to update nessecary location-based functions
end

function TrackingCombat:OnDisable()
	activeArea = false
	self:UnregisterEvent('PLAYER_REGEN_ENABLED')
	self:UnregisterEvent('PLAYER_REGEN_DISABLED')
	self:UnregisterEvent('PLAYER_ENTERING_WORLD')
	self:UnregisterEvent('PLAYER_TARGET_CHANGED')
	self:UnregisterEvent('COMBAT_LOG_EVENT_UNFILTERED') -- just to make sure its gone as its expensive
end

function TrackingCombat:CleanUpMusic()
	ShivvAudio:MusicStop('shivv-bg-combat')
	ShivvAudio:MusicStop('shivv-outside-combat')
	ShivvAudio:MusicStop('shivv-outside-elite')
	ShivvAudio:MusicStop('shivv-party-combat')
	ShivvAudio:MusicStop('shivv-party-boss')
	ShivvAudio:MusicStop('shivv-raid-combat')
	ShivvAudio:MusicStop('shivv-raid-boss')

	if (db.tieredCombat) then
		ShivvAudio:MusicStop('shivv-outside-t1')
		ShivvAudio:MusicStop('shivv-outside-t2')
		ShivvAudio:MusicStop('shivv-outside-t3')
		ShivvAudio:MusicStop('shivv-party-t1')
		ShivvAudio:MusicStop('shivv-party-t2')
		ShivvAudio:MusicStop('shivv-party-t3')
	end
end

function TrackingCombat:DBMOnPull()
	if (not activeArea) then return end -- don't trigger if not in an active area

	inDBMFight, inShivvFight = true, false

	-- cancel previous setup
	TrackingCombat:UnregisterEvent('COMBAT_LOG_EVENT_UNFILTERED')

	ShivvAudio:MusicStop('shivv-' .. activeAreaID .. '-combat')

	if (inDungeon and db.tieredCombat) then
		ShivvAudio:MusicStop('shivv-' .. activeAreaID .. '-t1')
		ShivvAudio:MusicStop('shivv-' .. activeAreaID .. '-t2')
		ShivvAudio:MusicStop('shivv-' .. activeAreaID .. '-t3')
	end

	ShivvAudio:MusicStart('shivv-' .. activeAreaID .. '-boss')
end

function TrackingCombat:DBMOnWipe()
	if (not inDBMFight) then return end -- only trigger if in a tracked DBM fight

	inDBMFight = false

	ShivvAudio:MusicStop('shivv-party-boss')
	ShivvAudio:MusicStop('shivv-raid-boss')
	ShivvAudio:SoundHandler(db.media[activeAreaID .. 'BossWipe'])
end

function TrackingCombat:DBMOnKill()
	if (not inDBMFight) then return end -- only trigger if in a tracked DBM fight

	inDBMFight = false

	ShivvAudio:MusicStop('shivv-party-boss')
	ShivvAudio:MusicStop('shivv-raid-boss')
	ShivvAudio:SoundHandler(db.media[activeAreaID .. 'BossKill'])
end

-- EVENT HANDLERS
function TrackingCombat:PLAYER_REGEN_DISABLED()
	if (inDBMFight or not activeArea or inShivvFight) then return end -- do nothing if DBM is handling this fight or if we're in an inactive area (or we already started combat)

	inShivvFight = true

	ShivvAudio:SoundHandler(db.media[activeAreaID .. 'Start'])

	if ((inOverworld or inDungeon) and db.tieredCombat) then
		activeMobs, inTier = 0, 1

		for m in pairs(knownMobs) do knownMobs[m] = nil end
	end

	if (inOverworld) then
		self:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED', 'TieredCLEU')

		if (db.tieredCombat) then
			ShivvAudio:MusicStart('shivv-outside-t1')
		else
			ShivvAudio:MusicStart('shivv-outside-combat')
		end
	elseif (inDungeon and db.tieredCombat) then
		self:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED', 'TieredCLEU')
		ShivvAudio:MusicStart('shivv-party-t1')
	else -- none of the above, do basic combat start
		ShivvAudio:MusicStart('shivv-' .. activeAreaID .. '-combat')
	end
end

function TrackingCombat:PLAYER_REGEN_ENABLED()
	if (inDBMFight or not inShivvFight) then return end -- do nothing if DBM is handling this fight, or we're not tracking a fight

	inShivvFight = false

	ShivvAudio:SoundHandler(db.media[activeAreaID .. 'End'])

	if (inDungeon or inOverworld) then -- in a dungeon or outside
		self:UnregisterEvent('COMBAT_LOG_EVENT_UNFILTERED') -- just to make sure its stopped
	end

	self:CleanUpMusic()
end

function TrackingCombat:PLAYER_ENTERING_WORLD()
	local inInstance, instanceType = IsInInstance()

	activeArea = false -- tracks whether the current 'area' is active
	inDungeon, inOverworld = false, false -- tracks special case areas

	if (inInstance) then
		self:UnregisterEvent('PLAYER_TARGET_CHANGED') -- kill target event (only used outside)

		if (instanceType == 'pvp' and db.bgEnable) then
			activeArea, activeAreaID = true, 'bg'
		elseif (instanceType == 'party' and db.partyEnable) then
			activeArea, activeAreaID = true, 'party'
			inDungeon = true
		elseif (instanceType == 'raid' and db.raidEnable) then
			activeArea, activeAreaID = true, 'raid'
		end
	else
		if (db.outsideEnable) then
			activeArea, activeAreaID = true, 'outside'
			inOverworld = true

			self:RegisterEvent('PLAYER_TARGET_CHANGED')
		end
	end
end

do
	local eliteTypes = {
		['worldboss'] = true,
		['rareelite'] = true,
		['elite'] = true,
		['rare'] = true
	}
	local scanUnits = {
		['target'] = true,
		['pettarget'] = true,
		['party1target'] = true,
		['party2target'] = true,
		['party3target'] = true,
		['party4target'] = true,
	}
	local UnitExists, UnitClassification, UnitGUID = _G.UnitExists, _G.UnitClassification, _G.UnitGUID

	function TrackingCombat:PLAYER_TARGET_CHANGED()
		for unit in pairs(scanUnits) do
			if (UnitExists(unit) and eliteTypes[UnitClassification(unit)]) then
				eliteMobs[tonumber(strsub(UnitGUID(unit), 7, 10), 16)] = true
			end
		end
	end
end

do
	local checkedEvents = {
		['RANGE_DAMAGE'] = true,
		['RANGE_MISSED'] = true,
		['SPELL_DAMAGE'] = true,
		['SPELL_MISSED'] = true,
		['SPELL_CAST_SUCCESS'] = true,
		['SPELL_PERIODIC_DAMAGE'] = true,
		['SWING_DAMAGE'] = true,
		['SWING_MISSED'] = true,
	}
	local IS_FRIEND = COMBATLOG_OBJECT_AFFILIATION_MINE + COMBATLOG_OBJECT_AFFILIATION_PARTY + COMBATLOG_OBJECT_AFFILIATION_RAID
	local IS_HOSTILENPC = bit.bor(COMBATLOG_OBJECT_TYPE_NPC, COMBATLOG_OBJECT_CONTROL_NPC, COMBATLOG_OBJECT_REACTION_HOSTILE) --0x00000240
	local IS_NULLGUID = 0x0000000000000000
	local band, strsub = bit.band, string.sub
	local hostileMob

	function TrackingCombat:TieredCLEU(cleu, time, evt, hc, sg, sn, sf, sf2, dg, dn, df)
		if (not checkedEvents[evt] or not dn) then return end

		hostileMob = nil

		if (band(sf, IS_FRIEND) ~= 0 and band(df, IS_HOSTILENPC) ~= 0) then
			hostileMob = dg
		elseif (band(sf, IS_HOSTILENPC) ~= 0 and band(df, IS_FRIEND) ~= 0) then
			hostileMob = sg
		end

		if (hostileMob == nil or hostileMob == IS_NULLGUID) then return end

		if (inOverworld and eliteMobs[tonumber(strsub(hostileMob, 7, 10), 16)]) then -- found an elite and we're fighting it!
			ShivvAudio:MusicStart('shivv-outside-elite')
			self:UnregisterEvent('COMBAT_LOG_EVENT_UNFILTERED') -- an elite fight is the 'highest' tier so stop tracking
		end

		if (not db.tieredCombat or inTier == 3 or knownMobs[hostileMob]) then return end -- abort if not using tiered combat (or in tier 3), or if the mob is known

		knownMobs[hostileMob] = true
		activeMobs = activeMobs + 1

		if (activeMobs > db[activeAreaID .. 'Tier3Mobs'] and inTier < 3) then -- enough mobs to trigger t2, and not yet playing
			inTier = 3
			ShivvAudio:MusicStart('shivv-' .. activeAreaID .. '-t3')
		elseif (activeMobs > db[activeAreaID .. 'Tier2Mobs'] and inTier < 2) then -- enough mobs to trigger t2, and not yet playing
			inTier = 2
			ShivvAudio:MusicStart('shivv-' .. activeAreaID .. '-t2')
		end
	end
end
